@@ -31,9 +31,11 @@ namespace Banshee.Collection
     public interface IBasicTrackInfo
     {
         string ArtistName { get; }
+        string AlbumArtist { get; }
         string AlbumTitle { get; }
         string TrackTitle { get; }
         string ArtworkId { get; }
+        Banshee.Base.SafeUri Uri { get; }
         TrackMediaAttributes MediaAttributes { get; }
     }
 }
