@@ -30,7 +30,7 @@ using System;
 
 namespace Banshee.Preferences
 {
-    public abstract class Root
+    public abstract class Root : IComparable
     {    
         private string id;
         private string name;
@@ -40,6 +40,8 @@ namespace Banshee.Preferences
         private bool visible;
         private object display_widget;
         private object mnemonic_widget;
+
+        public event Action<Root> Changed;
                 
         public Root ()
         {
@@ -47,6 +49,16 @@ namespace Banshee.Preferences
             visible = true;
         }
         
+        public int CompareTo (object o)
+        {
+            Root r = o as Root;
+            if (r == null) {
+                return -1;
+            }
+            
+            return Order.CompareTo (r.Order);
+        }
+
         public string Id {
             get { return id; }
             set { id = value; }
@@ -67,14 +79,20 @@ namespace Banshee.Preferences
             set { order = value; }
         }
         
-        public bool Sensitive {
+        public virtual bool Sensitive {
             get { return sensitive; }
-            set { sensitive = value; }
+            set {
+                sensitive = value;
+                OnChanged ();
+            }
         }
         
-        public bool Visible {
+        public virtual bool Visible {
             get { return visible; }
-            set { visible = value; }
+            set {
+                visible = value;
+                OnChanged ();
+            }
         }
         
         public virtual object DisplayWidget {
@@ -86,5 +104,13 @@ namespace Banshee.Preferences
             get { return mnemonic_widget ?? DisplayWidget; }
             set { mnemonic_widget = value; }
         }
+
+        protected void OnChanged ()
+        {
+            Action<Root> handler = Changed;
+            if (handler != null) {
+                handler (this);
+            }
+        }
     }
 }
