@@ -32,6 +32,8 @@ namespace Banshee.ServiceStack
 {
     public abstract class Client : IDisposable
     {
+        public event Action<Client> Started;
+
         public Client ()
         {
         }
@@ -43,5 +45,20 @@ namespace Banshee.ServiceStack
         public abstract string ClientId {
             get; 
         }
+
+        private bool is_started;
+        public bool IsStarted {
+            get { return is_started; }
+        }
+
+        protected void OnStarted ()
+        {
+            is_started = true;
+            Hyena.Log.InformationFormat ("{0} Client Started", ClientId);
+            Action<Client> handler = Started;
+            if (handler != null) {
+                handler (this);
+            }
+        }
     }
 }
