@@ -58,15 +58,7 @@ namespace Banshee.Sources
         protected DatabaseTrackListModel track_model;
         protected DatabaseAlbumListModel album_model;
         protected DatabaseArtistListModel artist_model;
-        
-        private DatabaseQueryFilterModel<string> genre_model;
-        protected DatabaseQueryFilterModel<string> GenreModel {
-            get {
-                return genre_model ?? 
-                    genre_model = new Banshee.Collection.Database.DatabaseQueryFilterModel<string> (this, DatabaseTrackModel, ServiceManager.DbConnection,
-                        Catalog.GetString ("All Genres ({0})"), UniqueId, BansheeQuery.GenreField, "Genre");
-            }
-        }
+        protected DatabaseQueryFilterModel<string> genre_model;
 
         private RateLimiter reload_limiter;
 
@@ -106,17 +98,15 @@ namespace Banshee.Sources
         }
 
         public DatabaseTrackListModel DatabaseTrackModel {
-            get {
-                return track_model ?? track_model = new DatabaseTrackListModel (ServiceManager.DbConnection, TrackProvider, this);
-            }
+            get { return track_model ?? (track_model = (Parent as DatabaseSource ?? this).CreateTrackModelFor (this)); }
             protected set { track_model = value; }
         }
 
         private IDatabaseTrackModelCache track_cache;
         protected IDatabaseTrackModelCache TrackCache {
             get {
-                return track_cache ?? track_cache = new DatabaseTrackModelCache<DatabaseTrackInfo> (
-                    ServiceManager.DbConnection, UniqueId, DatabaseTrackModel, TrackProvider);
+                return track_cache ?? (track_cache = new DatabaseTrackModelCache<DatabaseTrackInfo> (
+                    ServiceManager.DbConnection, UniqueId, DatabaseTrackModel, TrackProvider));
             }
             set { track_cache = value; }
         }
@@ -131,22 +121,42 @@ namespace Banshee.Sources
             
             current_filters_schema = CreateSchema<string[]> ("current_filters");
 
-            if (HasArtistAlbum) {
-                artist_model = new DatabaseArtistListModel (this, DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
-                album_model = new DatabaseAlbumListModel (this, DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
-                
-                AvailableFilters.Add (GenreModel);
-                AvailableFilters.Add (artist_model);
-                AvailableFilters.Add (album_model);
-                
-                DefaultFilters.Add (GenreModel);
-                DefaultFilters.Add (artist_model);
-                DefaultFilters.Add (album_model);
+            DatabaseSource filter_src = Parent as DatabaseSource ?? this;
+            foreach (IFilterListModel filter in filter_src.CreateFiltersFor (this)) {
+                AvailableFilters.Add (filter);
+                DefaultFilters.Add (filter);
             }
 
             reload_limiter = new RateLimiter (RateLimitedReload);
         }
 
+        protected virtual DatabaseTrackListModel CreateTrackModelFor (DatabaseSource src)
+        {
+            return new DatabaseTrackListModel (ServiceManager.DbConnection, TrackProvider, src);
+        }
+
+        protected virtual IEnumerable<IFilterListModel> CreateFiltersFor (DatabaseSource src)
+        {
+            if (!HasArtistAlbum) {
+                yield break;
+            }
+
+            DatabaseArtistListModel artist_model = new DatabaseArtistListModel (src, src.DatabaseTrackModel, ServiceManager.DbConnection, src.UniqueId);
+            DatabaseAlbumListModel album_model = new DatabaseAlbumListModel (src, src.DatabaseTrackModel, ServiceManager.DbConnection, src.UniqueId);
+            DatabaseQueryFilterModel<string> genre_model = new DatabaseQueryFilterModel<string> (src, src.DatabaseTrackModel, ServiceManager.DbConnection,
+                        Catalog.GetString ("All Genres ({0})"), src.UniqueId, BansheeQuery.GenreField, "Genre");
+
+            if (this == src) {
+                this.artist_model = artist_model;
+                this.album_model = album_model;
+                this.genre_model = genre_model;
+            }
+
+            yield return artist_model;
+            yield return album_model;
+            yield return genre_model;
+        }
+
         protected virtual void AfterInitialized ()
         {
             DatabaseTrackModel.Initialize (TrackCache);
@@ -276,6 +286,11 @@ namespace Banshee.Sources
         {
             return CanAddTracks && source != this;
         }
+
+        public override bool AcceptsUserInputFromSource (Source source)
+        {
+            return base.AcceptsUserInputFromSource (source) && CanAddTracks;
+        }
                 
         public override bool HasViewableTrackProperties {
             get { return true; }
@@ -291,13 +306,13 @@ namespace Banshee.Sources
         
         private IList<IFilterListModel> available_filters;
         public IList<IFilterListModel> AvailableFilters {
-            get { return available_filters ?? available_filters = new List<IFilterListModel> (); }
+            get { return available_filters ?? (available_filters = new List<IFilterListModel> ()); }
             protected set { available_filters = value; }
         }
         
         private IList<IFilterListModel> default_filters;
         public IList<IFilterListModel> DefaultFilters {
-            get { return default_filters ?? default_filters = new List<IFilterListModel> (); }
+            get { return default_filters ?? (default_filters = new List<IFilterListModel> ()); }
             protected set { default_filters = value; }
         }
         
@@ -640,14 +655,14 @@ namespace Banshee.Sources
         protected HyenaSqliteCommand prune_command;
         protected HyenaSqliteCommand PruneCommand {
             get {
-                return prune_command ?? prune_command = new HyenaSqliteCommand (String.Format (
+                return prune_command ?? (prune_command = new HyenaSqliteCommand (String.Format (
                         @"DELETE FROM CoreCache WHERE ModelID = ? AND ItemID NOT IN (SELECT ArtistID FROM CoreTracks WHERE TrackID IN (SELECT {0}));
                           DELETE FROM CoreCache WHERE ModelID = ? AND ItemID NOT IN (SELECT AlbumID FROM CoreTracks WHERE TrackID IN (SELECT {0}));",
                         track_model.TrackIdsSql
                     ),
                     artist_model.CacheId, artist_model.CacheId, 0, artist_model.Count,
                     album_model.CacheId, album_model.CacheId, 0, album_model.Count
-                );
+                ));
             }
         }
 
