@@ -36,23 +36,26 @@ namespace Banshee.Collection.Gui
     {
         public ColumnCellDuration (string property, bool expand) : base (property, expand)
         {
+            Alignment = Pango.Alignment.Right;
+            SetMinMaxStrings (TimeSpan.FromHours (5.73), TimeSpan.FromDays (7.34));
         }
         
-        protected override string Text {
-            get {
-                if (!(BoundObject is TimeSpan)) {
-                    return base.Text;
-                }
-                
-                // Fancy rounding commented out since it's not consistent with what is
-                // done in libbanshee.  See http://bugzilla.gnome.org/show_bug.cgi?id=520648
-                //int seconds = (int)Math.Round(((TimeSpan)BoundObject).TotalSeconds);
-                int seconds = (int) ((TimeSpan)BoundObject).TotalSeconds;
-                
-                return seconds >= 3600 ? 
-                    String.Format ("{0}:{1:00}:{2:00}", seconds / 3600, (seconds / 60) % 60, seconds % 60) :
-                    String.Format ("{0}:{1:00}", seconds / 60, seconds % 60);
+        protected override string GetText (object obj)
+        {
+            if (!(obj is TimeSpan)) {
+                return base.GetText (obj);
             }
+            
+            // Fancy rounding commented out since it's not consistent with what is
+            // done in libbanshee.  See http://bugzilla.gnome.org/show_bug.cgi?id=520648
+            //int seconds = (int)Math.Round(((TimeSpan)obj).TotalSeconds);
+            int seconds = (int) ((TimeSpan)obj).TotalSeconds;
+            
+            if (seconds == 0) {
+                return String.Empty;
+            }
+                
+            return Banshee.Sources.DurationStatusFormatters.ConfusingPreciseFormatter (TimeSpan.FromSeconds (seconds));
         }
     }
 }
