@@ -41,6 +41,10 @@ namespace Banshee.Gui.Widgets
     public class LargeTrackInfoDisplay : TrackInfoDisplay
     {
         private Gdk.Rectangle text_alloc;
+        private Dictionary<ImageSurface, Surface> surfaces = new Dictionary<ImageSurface, Surface> ();
+        private Pango.Layout first_line_layout;
+        private Pango.Layout second_line_layout;
+        private Pango.Layout third_line_layout;
     
         public LargeTrackInfoDisplay ()
         {
@@ -81,27 +85,45 @@ namespace Banshee.Gui.Widgets
             base.OnSizeAllocated (allocation);
             QueueDraw ();
         }
-       
-        protected override void RenderCoverArt (Cairo.Context cr, Gdk.Pixbuf pixbuf)
+
+        protected override void OnThemeChanged ()
         {
-            if (pixbuf == null) {
+            if (first_line_layout != null) {
+                first_line_layout.Dispose ();
+                first_line_layout = null;
+            }
+            
+            if (second_line_layout != null) {
+                second_line_layout.Dispose ();
+                second_line_layout = null;
+            }
+            
+            if (third_line_layout != null) {
+                third_line_layout.Dispose ();
+                third_line_layout = null;
+            }
+        }
+       
+        protected override void RenderCoverArt (Cairo.Context cr, ImageSurface image)
+        {
+            if (image == null) {
                 return;
             }
             
             Gdk.Rectangle alloc = RenderAllocation;
             int asr = ArtworkSizeRequest;
-            int reflect = (int)(pixbuf.Height * 0.2);
-            int surface_w = pixbuf.Width;
-            int surface_h = pixbuf.Height + reflect;
+            int reflect = (int)(image.Height * 0.2);
+            int surface_w = image.Width;
+            int surface_h = image.Height + reflect;
             int x = alloc.X + alloc.Width - asr;
             int y = alloc.Y;
             
-            Surface surface = SurfaceLookup (pixbuf);
-            if (surface == null) {
-                surface = CreateSurfaceForPixbuf (cr, pixbuf, reflect);
-                SurfaceCache (pixbuf, surface);
+            Surface scene = null;
+            if (!surfaces.TryGetValue (image, out scene)) {
+                scene = CreateScene (cr, image, reflect);
+                surfaces.Add (image, scene);
             }
-                
+
             cr.Rectangle (x, y, asr, alloc.Height);
             cr.Color = BackgroundColor;
             cr.Fill ();
@@ -109,30 +131,30 @@ namespace Banshee.Gui.Widgets
             x += (asr - surface_w) / 2;
             y += surface_h > asr ? 0 : (asr - surface_h) / 2;
             
-            cr.SetSource (surface, x, y);
+            cr.SetSource (scene, x, y);
             cr.Paint ();
         }
         
-        private Surface CreateSurfaceForPixbuf (Cairo.Context window_cr, Gdk.Pixbuf pixbuf, int reflect)
+        private Surface CreateScene (Cairo.Context window_cr, ImageSurface image, int reflect)
         {
             Surface surface = window_cr.Target.CreateSimilar (window_cr.Target.Content, 
-                pixbuf.Width, pixbuf.Height + reflect);
+                image.Width, image.Height + reflect);
             Cairo.Context cr = new Context (surface);
             
             cr.Save ();
             
-            Gdk.CairoHelper.SetSourcePixbuf (cr, pixbuf, 0, 0);
+            cr.SetSource (image);
             cr.Paint ();
             
-            cr.Rectangle (0, pixbuf.Height, pixbuf.Width, reflect);
+            cr.Rectangle (0, image.Height, image.Width, reflect);
             cr.Clip ();
             
             Matrix matrix = new Matrix ();
             matrix.InitScale (1, -1);
-            matrix.Translate (0, -(2 * pixbuf.Height) + 1);
+            matrix.Translate (0, -(2 * image.Height) + 1);
             cr.Transform (matrix);
             
-            Gdk.CairoHelper.SetSourcePixbuf (cr, pixbuf, 0, 0);
+            cr.SetSource (image);
             cr.Paint ();
             
             cr.Restore ();
@@ -140,11 +162,11 @@ namespace Banshee.Gui.Widgets
             Color bg_transparent = BackgroundColor;
             bg_transparent.A = 0.65;
             
-            LinearGradient mask = new LinearGradient (0, pixbuf.Height, 0, pixbuf.Height + reflect);
+            LinearGradient mask = new LinearGradient (0, image.Height, 0, image.Height + reflect);
             mask.AddColorStop (0, bg_transparent);
             mask.AddColorStop (1, BackgroundColor);
             
-            cr.Rectangle (0, pixbuf.Height, pixbuf.Width, reflect);
+            cr.Rectangle (0, image.Height, image.Width, reflect);
             cr.Pattern = mask;
             cr.Fill ();
             
@@ -161,6 +183,7 @@ namespace Banshee.Gui.Widgets
             Gdk.Rectangle alloc = RenderAllocation;
             int width = ArtworkSizeRequest;
             int fl_width, fl_height, sl_width, sl_height, tl_width, tl_height;
+            int pango_width = (int)(width * Pango.Scale.PangoScale);
 
             string first_line = GetFirstLineText (track);
             
@@ -176,19 +199,31 @@ namespace Banshee.Gui.Widgets
                     line.Substring (split_pos, line.Length - split_pos));
             }
 
+            if (first_line_layout == null) {
+                first_line_layout = CairoExtensions.CreateLayout (this, cr);
+                first_line_layout.Ellipsize = Pango.EllipsizeMode.End;
+                first_line_layout.Alignment = Pango.Alignment.Right;
+                
+                int base_size = first_line_layout.FontDescription.Size;
+                first_line_layout.FontDescription.Size = (int)(base_size * Pango.Scale.XLarge);
+            }
+            
+            if (second_line_layout == null) {
+                second_line_layout = CairoExtensions.CreateLayout (this, cr);
+                second_line_layout.Ellipsize = Pango.EllipsizeMode.End;
+                second_line_layout.Alignment = Pango.Alignment.Right;
+            }
+            
+            if (third_line_layout == null) {
+                third_line_layout = CairoExtensions.CreateLayout (this, cr);
+                third_line_layout.Ellipsize = Pango.EllipsizeMode.End;
+                third_line_layout.Alignment = Pango.Alignment.Right;
+            }
+
             // Set up the text layouts
-            Pango.Layout first_line_layout = null;
-            CairoExtensions.CreateLayout (this, cr, ref first_line_layout);
-            first_line_layout.Width = (int)(width * Pango.Scale.PangoScale);
-            first_line_layout.Ellipsize = Pango.EllipsizeMode.End;
-            first_line_layout.Alignment = Pango.Alignment.Right;
-
-            int base_size = first_line_layout.FontDescription.Size;
-                        
-            Pango.Layout second_line_layout = first_line_layout.Copy ();
-            Pango.Layout third_line_layout = first_line_layout.Copy ();
-
-            first_line_layout.FontDescription.Size = (int)(base_size * Pango.Scale.XLarge);
+            first_line_layout.Width = pango_width;
+            second_line_layout.Width = pango_width;
+            third_line_layout.Width = pango_width;
             
             // Compute the layout coordinates
             first_line_layout.SetMarkup (first_line);
@@ -213,9 +248,6 @@ namespace Banshee.Gui.Widgets
             }
 
             if (!renderArtistAlbum) {
-                first_line_layout.Dispose ();
-                second_line_layout.Dispose ();
-                third_line_layout.Dispose ();
                 return;
             }
             
@@ -224,15 +256,11 @@ namespace Banshee.Gui.Widgets
             
             cr.MoveTo (text_alloc.X, text_alloc.Y + fl_height + sl_height);
             PangoCairoHelper.ShowLayout (cr, third_line_layout);
-            
-            first_line_layout.Dispose ();
-            second_line_layout.Dispose ();
-            third_line_layout.Dispose ();
         }
         
         protected override void Invalidate ()
         {
-            if (CurrentPixbuf == null || CurrentTrack == null || IncomingPixbuf == null || IncomingTrack == null) {
+            if (CurrentImage == null || CurrentTrack == null || IncomingImage == null || IncomingTrack == null) {
                 QueueDraw ();
             } else {
                 Gdk.Rectangle alloc = RenderAllocation;
@@ -241,5 +269,14 @@ namespace Banshee.Gui.Widgets
                     alloc.Width - text_alloc.Width - Spacing, alloc.Height);
             }
         }
+        
+        protected override void InvalidateCache ()
+        {
+            foreach (Surface surface in surfaces.Values) {
+                surface.Destroy ();
+            }
+            
+            surfaces.Clear ();
+        }
     }
 }
