@@ -52,6 +52,10 @@ namespace Banshee.Gui
         }
         
         public event EventHandler TitleChanged;
+
+        protected BaseClientWindow (IntPtr ptr) : base (ptr)
+        {
+        }
     
         public BaseClientWindow (string title, string configNameSpace, int defaultWidth, int defaultHeight) : base (title)
         {
@@ -74,7 +78,12 @@ namespace Banshee.Gui
         
         public void ToggleVisibility ()
         {
-            if (Visible && IsActive) {
+            SetVisible (!(Visible && IsActive));
+        }
+
+        public void SetVisible (bool visible)
+        {
+            if (!visible) {
                 window_controller.Save ();
                 Visible = false;
             } else {
