@@ -56,6 +56,8 @@ namespace Banshee.Gui
             set { source_view = value; }
         }
 
+        public event Action<Source> Updated;
+
         public Source ActionSource {
             get { return ((SourceView == null) ? null :  SourceView.HighlightedSource) ?? ActiveSource; }
         }
@@ -143,15 +145,17 @@ namespace Banshee.Gui
 
         private void HandleActiveSourceChanged (SourceEventArgs args)
         {
-            UpdateActions ();
-            
-            if (last_source != null) {
-                last_source.Updated -= HandleActiveSourceUpdated;
-            }
-            
-            if (ActiveSource != null) {
-                ActiveSource.Updated += HandleActiveSourceUpdated;
-            }
+            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                UpdateActions ();
+                
+                if (last_source != null) {
+                    last_source.Updated -= HandleActiveSourceUpdated;
+                }
+                
+                if (ActiveSource != null) {
+                    ActiveSource.Updated += HandleActiveSourceUpdated;
+                }
+            });
         }
 
         private void HandleEditMenuActivated (object sender, EventArgs args)
@@ -161,7 +165,9 @@ namespace Banshee.Gui
         
         private void HandleActiveSourceUpdated (object o, EventArgs args)
         {
-            UpdateActions (true);
+            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                UpdateActions (true);
+            });
         }
 
 #endregion
@@ -170,7 +176,7 @@ namespace Banshee.Gui
 
         private void OnNewPlaylist (object o, EventArgs args)
         {
-            PlaylistSource playlist = new PlaylistSource ("New Playlist", ActivePrimarySource.DbId);
+            PlaylistSource playlist = new PlaylistSource ("New Playlist", ActivePrimarySource);
             playlist.Save ();
             playlist.PrimarySource.AddChildSource (playlist);
             playlist.NotifyUser ();
@@ -220,7 +226,7 @@ namespace Banshee.Gui
         {
             UpdateActions ();
 
-            string path = ActionSource.Properties.GetString ("GtkActionPath") ?? "/SourceContextMenu";
+            string path = ActionSource.Properties.Get<string> ("GtkActionPath") ?? "/SourceContextMenu";
             Gtk.Menu menu = Actions.UIManager.GetWidget (path) as Menu;
             if (menu == null || menu.Children.Length == 0) {
                 SourceView.ResetHighlight ();
@@ -352,8 +358,9 @@ namespace Banshee.Gui
                 UpdateAction ("SourcePropertiesAction", source.HasProperties, true, source);
                 UpdateAction ("RefreshSmartPlaylistAction", smart_playlist != null && smart_playlist.CanRefresh, true, source);
 
-                UpdateAction ("NewPlaylistAction", primary_source != null && primary_source.SupportsPlaylists, true, source);
-                UpdateAction ("NewSmartPlaylistAction", primary_source != null && primary_source.SupportsPlaylists, true, source);
+                bool playlists_writable = primary_source != null && primary_source.SupportsPlaylists && !primary_source.PlaylistsReadOnly;
+                UpdateAction ("NewPlaylistAction", playlists_writable, true, source);
+                UpdateAction ("NewSmartPlaylistAction", playlists_writable, true, source);
                 /*UpdateAction ("NewSmartPlaylistFromSearchAction", (source is LibrarySource || source.Parent is LibrarySource),
                         !String.IsNullOrEmpty (source.FilterQuery), source);*/
                     
@@ -372,6 +379,11 @@ namespace Banshee.Gui
             if (source != null) {
                 UpdateAction ("SortChildrenAction", source.Children.Count > 1, true, null);
             }
+
+            Action<Source> handler = Updated;
+            if (handler != null) {
+                handler (source);
+            }
         }
 
         private static bool ConfirmUnmap (IUnmapableSource source)
