@@ -70,17 +70,21 @@ namespace Banshee.Daap
             }
             
             // Dispose any remaining child sources
-            foreach (KeyValuePair <string, DaapSource> kv in source_map) {
-                kv.Value.Disconnect (true);
-                kv.Value.Dispose ();
+            if (source_map != null) {
+                foreach (KeyValuePair <string, DaapSource> kv in source_map) {
+                    if (kv.Value != null) {
+                        kv.Value.Disconnect (true);
+                        kv.Value.Dispose ();
+                    }
+                }
+            
+                source_map.Clear ();
             }
             
             if (container != null) {
                 ServiceManager.SourceManager.RemoveSource (container, true);
                 container = null;
             }
-            
-            source_map.Clear ();
         }
         
         private void OnServiceFound (object o, ServiceArgs args)
@@ -128,6 +132,11 @@ namespace Banshee.Daap
         }
 
         public void DelayedInitialize ()
+        {
+            Application.RunTimeout (3*1000, DelayedInitializeTimeout);
+        }
+
+        public bool DelayedInitializeTimeout ()
         {
             // Add the source, even though its empty, so that the user sees the
             // plugin is enabled, just no child sources yet.
@@ -151,6 +160,7 @@ namespace Banshee.Daap
             } catch (Exception e) {
                 Hyena.Log.Exception ("Failed to start DAAP client", e);
             }
+            return false;
         }
         
         string IService.ServiceName {
