@@ -86,7 +86,8 @@ namespace Lastfm
         private int hard_failure_retry_sec = 60;
         
         private HttpWebRequest now_playing_post;
-        private string current_now_playing_uri;
+        private bool now_playing_started;
+        private string current_now_playing_data;
         private HttpWebRequest current_web_req;
         private IAsyncResult current_async_result;
         private State state;
@@ -169,8 +170,6 @@ namespace Lastfm
 
         private void StateTransitionHandler (object o, ElapsedEventArgs e)
         {
-            Log.DebugFormat ("State transition handler running; state: {0}, connected: {1}", state, connected);
-            
             // if we're not connected, don't bother doing anything involving the network.
             if (!connected) {
                 return;
@@ -191,11 +190,10 @@ namespace Lastfm
                 } else {
                     if (queue.Count > 0 && session_id != null) {
                         state = State.NeedTransmit;
-                    } else if (current_now_playing_uri != null && session_id != null) {
+                    } else if (current_now_playing_data != null && session_id != null) {
                         // Now playing info needs to be sent
-                        NowPlaying (current_now_playing_uri);
+                        NowPlaying (current_now_playing_data);
                     } else {
-                        Hyena.Log.DebugFormat ("State transition handler going to sleep.");
                         StopTransitionHandler ();
                     }
                 }
@@ -265,8 +263,7 @@ namespace Lastfm
             state = State.WaitingForRequestStream;
             current_async_result = current_web_req.BeginGetRequestStream (TransmitGetRequestStream, ts);
             if (!(current_async_result.AsyncWaitHandle.WaitOne (TIME_OUT, false))) {
-		        Hyena.Log.Warning ("Audioscrobbler upload failed", 
-                                             "The request timed out and was aborted", false);
+                Log.Warning ("Audioscrobbler upload failed", "The request timed out and was aborted", false);
                 next_interval = DateTime.Now + new TimeSpan (0, 0, RETRY_SECONDS);
                 hard_failures++;
                 state = State.Idle;
@@ -312,7 +309,7 @@ namespace Lastfm
                 resp = current_web_req.EndGetResponse (ar);
             }
             catch (Exception e) {
-                Hyena.Log.Warning (String.Format("Failed to get the response: {0}", e), false);
+                Log.Warning (String.Format("Failed to get the response: {0}", e), false);
 
                 state = State.Idle;
                 next_interval = DateTime.Now + new TimeSpan (0, 0, RETRY_SECONDS);
@@ -331,7 +328,7 @@ namespace Lastfm
             DateTime now = DateTime.Now;
             if (line.StartsWith ("FAILED")) {
                 if (now - last_upload_failed_logged > TimeSpan.FromMinutes(FAILURE_LOG_MINUTES)) {
-                    Hyena.Log.Warning ("Audioscrobbler upload failed", line.Substring ("FAILED".Length).Trim(), false);
+                    Log.Warning ("Audioscrobbler upload failed", line.Substring ("FAILED".Length).Trim(), false);
                     last_upload_failed_logged = now;
                 }
                 
@@ -341,7 +338,7 @@ namespace Lastfm
             }
             else if (line.StartsWith ("BADSESSION")) {
                 if (now - last_upload_failed_logged > TimeSpan.FromMinutes(FAILURE_LOG_MINUTES)) {
-                    Hyena.Log.Warning ("Audioscrobbler upload failed", "session ID sent was invalid", false);
+                    Log.Warning ("Audioscrobbler upload failed", "session ID sent was invalid", false);
                     last_upload_failed_logged = now;
                 }
                 
@@ -353,7 +350,7 @@ namespace Lastfm
             } else if (line.StartsWith ("OK")) {
                 /* if we've previously logged failures, be nice and log the successful upload. */
                 if (last_upload_failed_logged != DateTime.MinValue) {
-                    Hyena.Log.Debug ("Audioscrobbler upload succeeded");
+                    Log.Debug ("Audioscrobbler upload succeeded");
                     last_upload_failed_logged = DateTime.MinValue;
                 }
                 
@@ -366,7 +363,7 @@ namespace Lastfm
                 state = State.Idle;
             } else {
                 if (now - last_upload_failed_logged > TimeSpan.FromMinutes(FAILURE_LOG_MINUTES)) {
-                    Hyena.Log.Warning ("Audioscrobbler upload failed", String.Format ("Unrecognized response: {0}", line), false);
+                    Log.Warning ("Audioscrobbler upload failed", String.Format ("Unrecognized response: {0}", line), false);
                     last_upload_failed_logged = now;
                 }
                 
@@ -420,7 +417,7 @@ namespace Lastfm
                 resp = current_web_req.EndGetResponse (ar);
             }
             catch (Exception e) {
-                Hyena.Log.Warning ("Failed to handshake: {0}", e.ToString (), false);
+                Log.Warning ("Failed to handshake: {0}", e.ToString (), false);
 
                 // back off for a time before trying again
                 state = State.Idle;
@@ -436,18 +433,17 @@ namespace Lastfm
 
             line = sr.ReadLine ();
             if (line.StartsWith ("BANNED")) {
-                Hyena.Log.Warning ("Audioscrobbler sign-on failed", "Player is banned", false);
+                Log.Warning ("Audioscrobbler sign-on failed", "Player is banned", false);
                                    
             } else if (line.StartsWith ("BADAUTH")) {
-                Hyena.Log.Warning ("Audioscrobbler sign-on failed", Catalog.GetString ("Last.fm username or password is invalid."));
+                Log.Warning ("Audioscrobbler sign-on failed", Catalog.GetString ("Last.fm username or password is invalid."));
                 LastfmCore.Account.CryptedPassword = null;
             } else if (line.StartsWith ("BADTIME")) {
-                Hyena.Log.Warning ("Audioscrobbler sign-on failed", 
+                Log.Warning ("Audioscrobbler sign-on failed", 
                                                   "timestamp provided was not close enough to the current time", false);
             } else if (line.StartsWith ("FAILED")) {
-                Hyena.Log.Warning ("Audioscrobbler sign-on failed",
-                                                  String.Format ("Temporary server failure: {0}",
-                                                                  line.Substring ("FAILED".Length).Trim()), false);
+                Log.Warning ("Audioscrobbler sign-on failed",
+                    String.Format ("Temporary server failure: {0}", line.Substring ("FAILED".Length).Trim ()), false);
                 hard_failure = true;
             } else if (line.StartsWith ("OK")) {
                 success = true;
@@ -500,42 +496,42 @@ namespace Lastfm
             // Fall back to prefixing the URL with a # in case we haven't actually
             // authenticated yet. We replace it with the now_playing_url and session_id
             // later on in NowPlaying(uri).
-            string uriprefix = "#";
+            string dataprefix = "#";
             
             if (session_id != null) {
-                uriprefix = String.Format ("{0}?s={1}", now_playing_url, session_id);
+                dataprefix = String.Format ("s={0}", session_id);
             }
             
-            string uri = String.Format ("{0}&a={1}&t={2}&b={3}&l={4}&n={5}&m={6}",
-                                        uriprefix,
+            string data = String.Format ("{0}&a={1}&t={2}&b={3}&l={4}&n={5}&m={6}",
+                                        dataprefix,
                                         HttpUtility.UrlEncode(artist),
                                         HttpUtility.UrlEncode(title),
-			                            HttpUtility.UrlEncode(album),
+                                        HttpUtility.UrlEncode(album),
                                         duration.ToString(),
                                         str_track_number,
-			                            mbrainzid);
+                                        mbrainzid);
 
-            NowPlaying (uri);
+            Console.WriteLine ("Submitting via non-uri handler.");            
+            NowPlaying (data);
         }
         
-        private void NowPlaying (string uri)
+        private void NowPlaying (string data)
         {            
-            if (now_playing_post != null) {
-                Hyena.Log.DebugFormat ("Now-playing submission already started - aborting.");
-                now_playing_post.Abort ();
+            if (now_playing_started) {
+                return;
             }
             
             // If the URI begins with #, then we know the URI was created before we
             // had actually authenticated. So, because we didn't know the session_id and
             // now_playing_url previously, we should now, so we put that in and create our
             // new URI.
-            if (uri.StartsWith ("#") && session_id != null) {
-                uri = String.Format ("{0}?s={1}{2}", now_playing_url,
-                                    session_id,
-                                    uri.Substring (1));
+            if (data.StartsWith ("#") && session_id != null) {
+                data = String.Format ("s={0}{1}",
+                                      session_id,
+                                      data.Substring (1));
             }
             
-            current_now_playing_uri = uri;
+            current_now_playing_data = data;
             
             if (session_id == null) {
                 // Go connect - we'll come back later in main timer loop.
@@ -544,18 +540,40 @@ namespace Lastfm
             }
 
             try {
-                now_playing_post = (HttpWebRequest) WebRequest.Create (uri);
+                now_playing_post = (HttpWebRequest) WebRequest.Create (now_playing_url);
                 now_playing_post.UserAgent = LastfmCore.UserAgent;
                 now_playing_post.Method = "POST";
                 now_playing_post.ContentType = "application/x-www-form-urlencoded";
-                now_playing_post.ContentLength = uri.Length;
-                now_playing_post.BeginGetResponse (NowPlayingGetResponse, null);
+
+                if (state == State.Idle) {
+                    // Don't actually POST it until we're idle (that is, we
+                    // probably have stuff queued which will reset the Now
+                    // Playing if we send them first).
+                    now_playing_started = true;
+                    now_playing_post.BeginGetRequestStream (NowPlayingGetRequestStream, data);
+                }
             } catch (Exception ex) {
-                Hyena.Log.Warning ("Audioscrobbler NowPlaying failed",
+                Log.Warning ("Audioscrobbler NowPlaying failed",
                                   String.Format ("Exception while creating request: {0}", ex), false);
                 
-                // Reset current_now_playing_uri if it was the problem.
-                current_now_playing_uri = null;
+                // Reset current_now_playing_data if it was the problem.
+                current_now_playing_data = null;
+            }
+        }
+
+        private void NowPlayingGetRequestStream (IAsyncResult ar)
+        {
+            try {
+                string data = ar.AsyncState as string;
+                ASCIIEncoding encoding = new ASCIIEncoding ();
+                byte[] data_bytes = encoding.GetBytes (data);
+                Stream request_stream = now_playing_post.EndGetRequestStream (ar);
+                request_stream.Write (data_bytes, 0, data_bytes.Length);
+                request_stream.Close ();
+
+                now_playing_post.BeginGetResponse (NowPlayingGetResponse, null);
+            } catch (Exception e) {
+                Log.Exception (e);
             }
         }
 
@@ -569,11 +587,11 @@ namespace Lastfm
 
                 string line = sr.ReadLine ();
                 if (line == null) {
-                    Hyena.Log.Warning ("Audioscrobbler NowPlaying failed", "No response", false);
+                    Log.Warning ("Audioscrobbler NowPlaying failed", "No response", false);
                 }
                 
                 if (line.StartsWith ("BADSESSION")) {
-                    Hyena.Log.Warning ("Audioscrobbler NowPlaying failed", "Session ID sent was invalid", false);
+                    Log.Warning ("Audioscrobbler NowPlaying failed", "Session ID sent was invalid", false);
                     // attempt to re-handshake on the next interval
                     session_id = null;
                     next_interval = DateTime.Now + new TimeSpan (0, 0, RETRY_SECONDS);
@@ -582,26 +600,28 @@ namespace Lastfm
                     return;
                 } else if (line.StartsWith ("OK")) {
                     // NowPlaying submitted  
-                    Hyena.Log.DebugFormat ("Submitted NowPlaying track to Audioscrobbler");
+                    Log.DebugFormat ("Submitted NowPlaying track to Audioscrobbler");
+                    now_playing_started = false;
                     now_playing_post = null;
-                    current_now_playing_uri = null;
+                    current_now_playing_data = null;
                     return;
                 } else {
-                    Hyena.Log.Warning ("Audioscrobbler NowPlaying failed", "Unexpected or no response", false);       
+                    Log.Warning ("Audioscrobbler NowPlaying failed", "Unexpected or no response", false);       
                 }
             }
             catch (Exception e) {
-                Hyena.Log.Warning ("Audioscrobbler NowPlaying failed", 
-                              String.Format("Failed to post NowPlaying: {0}", e), false);
+                Log.Warning ("Audioscrobbler NowPlaying failed", 
+                    String.Format("Failed to post NowPlaying: {0}", e), false);
             }
             
             // NowPlaying error/success is non-crutial.
             hard_failures++;
             if (hard_failures < 3) {
-                NowPlaying (current_now_playing_uri);
+                NowPlaying (current_now_playing_data);
             } else {
                 // Give up - NowPlaying status information is non-critical.
-                current_now_playing_uri = null;
+                current_now_playing_data = null;
+                now_playing_started = false;
                 now_playing_post = null;
             }
         }
